using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Windows.Media;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.AClasses;
using ff14bot.Behavior;
using TreeSharp;
using System.Windows.Forms;
using Buddy.Coroutines;
using ff14bot.RemoteAgents;
using ff14bot.RemoteWindows;
using Action = TreeSharp.Action;

namespace ChocoboTrainer
{

    public enum ChocoboFood : uint
    {
        Curiel_Root = 7894,
        Sylkis_Bud = 7895,
        Mimett_Gourd = 7897,
        Tantalplant = 7898,
        Pahsana_Fruit = 7900,
        Krakka_Root = 8165,

        Any
    }


    public class ChocoboTrainerSettings : JsonSettings
    {

        private static ChocoboTrainerSettings _settings;

        public static ChocoboTrainerSettings Instance
        {
            get { return _settings ?? (_settings = new ChocoboTrainerSettings()); }
        }

        public ChocoboTrainerSettings()
            : base(Path.Combine(CharacterSettingsDirectory, "ChocoboTrainerSettings.json"))
        {
        }




        private ChocoboFood _ChocoboFood;
        [Setting]
#if RB_CN
	[DisplayName("陆行鸟食物")]
	[Description("需要为陆行鸟喂食什么食物?")]
#else
	[DisplayName("ChocoboFood")]
	[Description("What food to fed the chocobos")]
#endif
        [DefaultValueAttribute(ChocoboFood.Krakka_Root)]
        public ChocoboFood ChocoboFood
        {
            get { return _ChocoboFood; }
            set
            {
                if (_ChocoboFood != value)
                {
                    _ChocoboFood = value;
                    Save();
                }

            }
        }

        private string _PlayerName;
        [Setting]
#if RB_CN
	[DisplayName("玩家名字")]
	[Description("需要训练谁的陆行鸟?")]
#else
	[DisplayName("PlayerName")]
	[Description("Whose chocobo to train?")]
#endif
        [DefaultValueAttribute("")]
        public string PlayerName
        {
            get { return _PlayerName; }
            set
            {
                if (_PlayerName != value)
                {
                    _PlayerName = value;
                    Save();
                }

            }
        }

    }

    public class ChocoboTrainer : BotBase
    {
        public override string Name
        {
            get 
			{ 
			#if RB_CB
				return "陆行鸟训练师";
			#else	
				return "ChocoboTrainer";
			#endif
			}
        }
        public override string EnglishName
        {
            get 
			{ 
				return "ChocoboTrainer";
			}
        }

        public override bool WantButton
        {
            get
            {
                return true;
            }
        }

        private SettingsForm settings;
        public override void OnButtonPress()
        {
            if (settings == null || settings.IsDisposed)
                settings = new SettingsForm();
            try
            {
                settings.Show();
                settings.Activate();
            }
            catch (ArgumentOutOfRangeException ee)
            {

            }
        }

        public override void Start()
        {
            Navigator.PlayerMover = new NullMover();
            Navigator.NavigationProvider = new NullProvider();

            Logging.Write("Starting ChocoboTrainer");
        }

        public override void Stop()
        {
        }

        public override PulseFlags PulseFlags
        {
            get { return PulseFlags.All; }
        }


        private async Task<bool> MainWorkerTask()
        {

            if (SelectString.IsOpen)
            {
                SelectString.ClickSlot(0);
                await Coroutine.Wait(5000, () => HousingChocoboList.IsOpen || HousingMyChocobo.IsOpen);
                //Give it sometime to populate data
                await Coroutine.Sleep(1500);
            }

            //Make sure we didn't just timeout above
            if (HousingChocoboList.IsOpen)
            {
                await HousingChocoboListWork();
            }
            else if (HousingMyChocobo.IsOpen)
            {
                await HousingMyChocoboWork();
            }
            else
            {
                TreeRoot.Stop("Failed to open menu");
            }

            return true;
        }
        /// <summary>
        /// This regex will extract the number of minutes a chocobo needs to rest for, the first group (?:.*?) isnt captured because of the ?: 
        /// it is considered 'lazy' because of the prepended ? meaning it will only match as few as characters as possible. 
        /// Otherwise the first group would match  'Train (read in 5' leaving only the last digit to be captured by the digit group
        /// </summary>
        internal static Regex TimeRegex = new Regex(@"(?:.*?)(\d+).*", RegexOptions.Compiled);
        private async Task HousingMyChocoboWork()
        {
            var matches = TimeRegex.Match(HousingMyChocobo.Lines[0]);
            if (!matches.Success)
            {
                //We are ready to train now
                HousingMyChocobo.SelectLine(0);

                Log("Waiting for inventory menu to appear....");
                //Wait for the inventory window to open and be ready
                //Technically the inventory windows are always 'open' so we check if their callbackhandler has been set
                if (!await Coroutine.Wait(5000, () => AgentInventoryContext.Instance.CallbackHandlerSet))
                {
                    Log("Inventorymenu failed to appear, aborting current iteration and starting over.");
                    return;
                }

                Log("Feeding Chocobo {0}", ChocoboTrainerSettings.Instance.ChocoboFood);
                AgentHousingBuddyList.Instance.Feed((uint)ChocoboTrainerSettings.Instance.ChocoboFood);

                Log("Waiting for cutscene to start....");
                if (await Coroutine.Wait(5000, () => QuestLogManager.InCutscene))
                {
                    Log("Waiting for cutscene to end....");
                    await Coroutine.Wait(Timeout.Infinite, () => !QuestLogManager.InCutscene);
                }

                Log("Waiting for menu to reappear....");
                await Coroutine.Wait(Timeout.Infinite, () => HousingMyChocobo.IsOpen);
                await Coroutine.Sleep(1000);

            }
            else
            {
                var timeToSleep = TimeSpan.FromMinutes(Int32.Parse(matches.Groups[1].Value) + 1);
                HousingMyChocobo.Close();
                Logging.Write(@"Sleeping for {0},until our chocobo is ready.", timeToSleep);
                await Coroutine.Sleep(timeToSleep);
            }

        }
        private readonly HashSet<string> _blacklistedNames = new HashSet<string>();
        private static void Log(string text, params object[] args)
        {
            var msg = string.Format("[ChocoboTrainer] " + text, args);
            Logging.Write(Colors.Green, msg);
        }
        private async Task HousingChocoboListWork()
        {
            //Look for our chocobo
            var items = HousingChocoboList.Items;
            var targetName = ChocoboTrainerSettings.Instance.PlayerName;

            //512 possible chocobos, 14 items per page...
            for (uint stableSection = 0; stableSection < AgentHousingBuddyList.Instance.TotalPages; stableSection++)
            {

                if (stableSection != AgentHousingBuddyList.Instance.CurrentPage)
                {
                    Log("Switching to page {0}", stableSection);
                    HousingChocoboList.SelectSection(stableSection);
                    await Coroutine.Sleep(5000);
                    items = HousingChocoboList.Items;
                }

                Scan:
                for (uint i = 0; i < items.Length; i++)
                {

                    if (string.IsNullOrEmpty(items[i].PlayerName))
                        continue;

                    if (!_blacklistedNames.Contains(items[i].PlayerName) && (string.Equals(items[i].PlayerName, targetName, StringComparison.OrdinalIgnoreCase) || string.IsNullOrEmpty(targetName)))
                    {
                        if (items[i].ReadyAt < DateTime.Now)
                        {
                            //Personal chocobo is handled differently
                            if (i == 0)
                            {
                                Log("Selecting my Chocobo", i, stableSection);
                                HousingChocoboList.SelectMyChocobo();
                            }
                            else
                            {
                                Log("Selecting Chocobo {0} on page {1}", i, stableSection);
                                HousingChocoboList.SelectChocobo(i);
                            }

                            //Chocobo is maxed out, don't interact with it again
                            if (await Coroutine.Wait(2500, () => SelectYesno.IsOpen))
                            {
                                Log("SelectYesNo window popped up, closing it and moving on");
                                SelectYesno.ClickNo();
                                _blacklistedNames.Add(items[i].PlayerName);
                                await Coroutine.Sleep(1000);
                                continue;
                            }


                            Log("Waiting for inventory menu to appear....");
                            //Wait for the inventory window to open and be ready
                            //Technically the inventory windows are always 'open' so we check if their callbackhandler has been set
                            if (!await Coroutine.Wait(5000, () => AgentInventoryContext.Instance.CallbackHandlerSet))
                            {
                                Log("Inventorymenu failed to appear, aborting current iteration and starting over.");
                                return;
                            }

                            Log("Feeding Chocobo {0}", ChocoboTrainerSettings.Instance.ChocoboFood);
                            AgentHousingBuddyList.Instance.Feed((uint)ChocoboTrainerSettings.Instance.ChocoboFood);

                            Log("Waiting for cutscene to start....");
                            if (await Coroutine.Wait(5000, () => QuestLogManager.InCutscene))
                            {
                                Log("Waiting for cutscene to end....");
                                await Coroutine.Wait(Timeout.Infinite, () => !QuestLogManager.InCutscene);
                            }

                            Log("Waiting for menu to reappear....");
                            await Coroutine.Wait(Timeout.Infinite, () => HousingChocoboList.IsOpen);
                            await Coroutine.Sleep(1000);
                            return;


                        }
                        else
                        {
                            //Don't sleep if we are feeding everyone
                            if (!string.IsNullOrEmpty(targetName))
                            {
                                //Boring, need to wait Zzz
                                var timeToSleep = (items[i].ReadyAt - DateTime.Now) + oneminute;
                                Logging.Write(@"Sleeping for {0},until our chocobo is ready.", timeToSleep);
                                await Coroutine.Sleep(timeToSleep);
								items = HousingChocoboList.Items;
                                goto Scan;
                            }

                        }
                    }
                }

                

            }
        }
        private TimeSpan oneminute = new TimeSpan(0,1,0);
        private Composite _root;
        public override Composite Root
        {
            get
            {
                return _root ?? (_root = new ActionRunCoroutine(r => MainWorkerTask()));
            }
        }

        public override bool RequiresProfile
        {
            get { return false; }
        }

    }
}
