﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Emit;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Objects;
using ff14bot.RemoteWindows;
using TreeSharp;

namespace Fertilizer
{
    public class Fertilizer : BotBase
    {
        public override string Name
        {
            get
            {
#if RB_CN
                return "栽培模式";
#else
                return "Gardening bot";
#endif
            }
        }

        public override bool WantButton
        {
            get { return true; }
        }


        private bool forceWater = false;
        public override void OnButtonPress()
        {
            forceWater = true;
        }

        public override string EnglishName
        {
            get { return "Gardening bot"; }
        }
        public override PulseFlags PulseFlags
        {
            get { return PulseFlags.All; }
        }

        public override bool RequiresProfile
        {
            get { return false; }
        }

        private static void Log(string text, params object[] args)
        {
            var msg = string.Format("[Gardening bot] " + text, args);
            Logging.Write(Colors.Green, msg);
        }

        private static void LogVerbose(string text, params object[] args)
        {
            var msg = string.Format("[Gardening bot] " + text, args);
            Logging.WriteVerbose(msg);
        }

        private Composite _root;
        public override void Start()
        {
            _root = new ActionRunCoroutine(r => Fertilize());
        }

        private readonly Dictionary<uint, int> FertilizedCount = new Dictionary<uint, int>();
        private readonly Dictionary<uint, DateTime> ReadyAt = new Dictionary<uint, DateTime>();


        private bool Ready(uint objectId)
        {
            DateTime time;
            ReadyAt.TryGetValue(objectId, out time);
            return time == default(DateTime) || DateTime.UtcNow > time;
        }
        private void IncreaseFertileCount(uint id)
        {
            int currentCount;
            FertilizedCount.TryGetValue(id, out currentCount);
            FertilizedCount[id] = currentCount + 1;
        }

        private int GetFertileCount(uint id)
        {
            int currentCount;
            FertilizedCount.TryGetValue(id, out currentCount);
            return currentCount;
        }
		
		private const int PostInteractDelay = 2300;

        private async Task<bool> Fertilize()
        {

            var NeedWatering = GardenManager.FlowersAndPlants.Where(r => !Blacklist.Contains(r) && r.Distance2D(Core.Player) < 5).ToArray();
            Log("{0} plants found nearby",NeedWatering.Count());
            //return false;
            foreach (var plant in NeedWatering)
            {
                //Water it if it needs it or if we have fertilized it 5 or more times.
                if (forceWater || GardenManager.NeedsWatering(plant) || GetFertileCount(plant.ObjectId) >= 5)
                {
                    var result = GardenManager.GetCrop(plant);
                    if (result != null)
                    {
                        Log("Watering {0} {1:X}", result, plant.ObjectId);
                        plant.Interact();
                        if (await Coroutine.Wait(5000, () => Talk.DialogOpen))
                        {
                            Talk.Next();

                            if (plant is InteractableHousingObject)
                            {
                                Log("Waiting for seconday dialog");
                                await Coroutine.Sleep(500);
                                if (await Coroutine.Wait(5000, () => Talk.DialogOpen))
                                {
                                    Talk.Next();
                                }
                            }


                            if (await Coroutine.Wait(5000, () => SelectString.IsOpen))
                            {
                                //Harvest drops it down to two
                                if (SelectString.LineCount == 4)
                                {
                                    SelectString.ClickSlot(1);
                                    FertilizedCount[plant.ObjectId] = 0;
                                    await Coroutine.Sleep(PostInteractDelay);
                                }
                                else
                                {
                                    Blacklist.Add(plant, BlacklistFlags.All, TimeSpan.FromMinutes(61),"Plant is ready to be harvested");
                                    SelectString.ClickSlot(2);
                                }
                            }
                        }
                        else
                        {
                            Log("GardenManager.GetCrop returned null {0:X}", plant.ObjectId);
                        }
                    }
                }
            }
            forceWater = false;

            var plants = GardenManager.FlowersAndPlants.Where(r => !Blacklist.Contains(r) && Ready(r.ObjectId) && r.Distance2D(Core.Player) < 5).ToArray();
            foreach (var plant in plants)
            {

                var result = GardenManager.GetCrop(plant);
                if (result != null)
                {
                    Log("Fertilizing {0} {1:X}", GardenManager.GetCrop(plant), plant.ObjectId);
                    plant.Interact();

                    if (await Coroutine.Wait(5000, () => Talk.DialogOpen))
                    {
                        Talk.Next();

                        if (plant is InteractableHousingObject)
                        {
                            if (await Coroutine.Wait(5000, () => Talk.DialogOpen))
                            {
                                Log("Waiting for seconday dialog");
                                await Coroutine.Sleep(500);
                                if (await Coroutine.Wait(5000, () => Talk.DialogOpen))
                                {
                                    Talk.Next();
                                }
                            }
                        }


                        if (await Coroutine.Wait(5000, () => SelectString.IsOpen))
                        {

                            //Harvest drops it down to two
                            if (SelectString.LineCount == 4)
                            {
                                SelectString.ClickSlot(0);
                                if (await Coroutine.Wait(2000, () => GardenManager.ReadyToFertilize))
                                {
                                    if (GardenManager.Fertilize() == FertilizeResult.Success)
                                    {
                                        LogVerbose("Plant with objectId {0:X} was fertilized, will be fertilized again in 61 minutes.", plant.ObjectId);
                                        ReadyAt[plant.ObjectId] = DateTime.UtcNow.AddMinutes(61);
                                        IncreaseFertileCount(plant.ObjectId);
                                        await Coroutine.Sleep(PostInteractDelay);
                                    }

                                }
                                else
                                {
                                    ReadyAt[plant.ObjectId] = DateTime.UtcNow.AddMinutes(11);
                                    LogVerbose("Plant with objectId {0:X} not able to be fertilized, trying again in 11 minutes.", plant.ObjectId);
                                }
                            }
                            else
                            {
                                Blacklist.Add(plant, BlacklistFlags.All, TimeSpan.FromMinutes(61), "Plant is ready to be harvested");
                                SelectString.ClickSlot(2);
                            } 
                            
                        }
                    }
                }



            }



            return false;
        }

        public override Composite Root
        {
            get { return _root; }
        }
    }
}
