using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.AClasses;
using ff14bot.Behavior;
using TreeSharp;
using System.Windows.Forms;
using Buddy.Coroutines;
using ff14bot.Objects;
using ff14bot.RemoteAgents;
using ff14bot.RemoteWindows;
using Action = TreeSharp.Action;

namespace RaidBro
{

    public class RaidBroSettings : JsonSettings
    {

        private static RaidBroSettings _settings;

        public static RaidBroSettings Instance
        {
            get { return _settings ?? (_settings = new RaidBroSettings()); }
        }

        public RaidBroSettings(): base(Path.Combine(CharacterSettingsDirectory, "RaidBroSettings.json"))
        {

        }

        private bool _overlay;

        [Description("Use the overlay to send messages when the bot is paused and resumed?")]
        [DefaultValue(true)] //shift +x
        public bool UseOverlay
        {
            get { return _overlay; }
            set
            {
                if (_overlay != value)
                {
                    _overlay = value;
                    Save();
                }

            }
        }

		private bool _skipCutScenes;
        [Description("Changes take effect after stopping and starting the bot")]
        [DefaultValue(false)]
        public bool SkipCutScenes
        {
            get { return _skipCutScenes; }
            set
            {
                if (_skipCutScenes != value)
                {
                    _skipCutScenes = value;
                    Save();
                }

            }
        }


        private bool _UsePartyInCombat;
        [Description("Checks if party memebers are incombat instead of only checking the player. Changes take effect after stopping and starting the bot")]
        [DefaultValue(true)]
        public bool CheckPartyMemberCombat
        {
            get { return _UsePartyInCombat; }
            set
            {
                if (_UsePartyInCombat != value)
                {
                    _UsePartyInCombat = value;
                    Save();
                }

            }
        }

        private Keys _pausekey;

        [Description("Changes take effect after stopping and starting the bot")]
        [DefaultSettingValue("65624")] //shift +x
        public Keys PauseKey
        {
            get { return _pausekey; }
            set
            {
                if (_pausekey != value)
                {
                    _pausekey = value;
                    Save();
                }

            }
        }


    }

    public class RaidBro : BotBase
    {
        public override string Name
        {
            get { return "Combat Assist"; }
        }


        public override bool WantButton
        {
            get
            {
                return true;
            }
        }

        private SettingsForm settings;
        public override void OnButtonPress()
        {
            if (settings == null || settings.IsDisposed)
                settings = new SettingsForm();
            try
            {
                settings.Show();
                settings.Activate();
            }
            catch (ArgumentOutOfRangeException ee)
            {

            }
        }

        public static Keys Key(Keys number)
        {
            return (Keys)((int)number & 0x0000FFFF);
        }
        public static ModifierKeys ModifierKey(Keys number)
        {
            var key = (Keys)((int)number & 0xFFFF0000);
            return (ModifierKeys)Enum.Parse(typeof(ModifierKeys), key.ToString());
        }

        public bool IsPaused;


        public void setupHotkey()
        {


            HotkeyManager.Register("Raidbro Pause", Key(RaidBroSettings.Instance.PauseKey), ModifierKey(RaidBroSettings.Instance.PauseKey),
            hk =>
            {
                IsPaused = !IsPaused;
                if (RaidBroSettings.Instance.UseOverlay)
                {
                    Core.OverlayManager.AddToast(IsPaused ? "Combat Assist Paused!" : "Combat Assist Resumed!", 2000);
                }
                
                Logging.Write(IsPaused ? "Combat Assist Paused!" : "Combat Assist Resumed!");
            });
        }


        private async Task<bool> HandleCutscenes()
        {
            if (QuestLogManager.InCutscene)
            {
                if (AgentCutScene.Instance.CanSkip && !SelectString.IsOpen)
                {
                    AgentCutScene.Instance.PromptSkip();
                    if (await Coroutine.Wait(600, () => SelectString.IsOpen))
                    {
                        SelectString.ClickSlot(0);
                        await Coroutine.Sleep(1000);
                    }
                }
            }
            return false;
        }


        private bool partyInCombat
        {
            get
            {
                foreach (var partyMember in PartyManager.VisibleMembers)
                {
                    if (partyMember.BattleCharacter.InCombat)
                    {
                        return true;
                    }
                }
                return false;
            }
        }


        public override void Start()
        {
            Navigator.PlayerMover = new NullMover();
            Navigator.NavigationProvider = new NullProvider();
            setupHotkey();


            CanRunDecoratorDelegate canRunDecoratorDelegate;
            if (RaidBroSettings.Instance.CheckPartyMemberCombat)
            {
                //Check if player is incombat first as its a much easier check vs the party status
                canRunDecoratorDelegate = r => (Core.Player.InCombat|| partyInCombat) && Core.Player.HasTarget;
            }
            else
            {
                canRunDecoratorDelegate = r => Core.Player.InCombat && Core.Player.HasTarget;
            }

            if (RaidBroSettings.Instance.SkipCutScenes)
            {
                _root = new Decorator(r => !IsPaused,new PrioritySelector(new ActionRunCoroutine(r=> HandleCutscenes()), new Decorator(canRunDecoratorDelegate, BrainBehavior.CombatLogic)));
            }
            else
            {
                _root = new Decorator(r => !IsPaused,new Decorator(canRunDecoratorDelegate, BrainBehavior.CombatLogic));
            }

            Logging.Write("Starting Combat Assist");
            GameSettingsManager.FaceTargetOnAction = false;
        }

        public override void Stop()
        {
            _root = null;
            HotkeyManager.Unregister("Raidbro Pause");
        }

        public override PulseFlags PulseFlags
        {
            get { return PulseFlags.All; }
        }

        private Composite _root;
        public override Composite Root
        {
            get { return _root; }
        }

        public override bool RequiresProfile
        {
            get { return false; }
        }

    }
}
