﻿//!CompilerOption:AddRef:SlimDx.dll

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using System.Windows.Navigation;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Interfaces;
using ff14bot.Managers;
using ff14bot.NeoProfiles;
using ff14bot.Objects;
using RB3DOverlay.Overlay;
using SlimDX.Direct3D9;
using Application = System.Windows.Forms.Application;
using Color = System.Drawing.Color;
using DrawingContext = RB3DOverlay.Overlay.DrawingContext;
using Matrix = SlimDX.Matrix;
using Vector3 = SlimDX.Vector3;
using NVector3 = Clio.Utilities.Vector3;

namespace RB3DOverlay
{
    public class Plugin : BotPlugin
    {
        private RenderForm _renderForm;

        public override void OnPulse()
        {
        }

        public override void OnInitialize()
        {
            //Check if rivatuner is running, if rivatuner is running and isn't blocked from attaching to rebornbuddy it can cause RB to crash
            var rivaTunerRunning = Process.GetProcessesByName("RTSS").Any();
            if (rivaTunerRunning)
            {
                Logging.Write(Colors.Red, @"Rivatuner has been detected running on this machine. If rebornbuddy crashes, add rebornbuddy.exe to rivatuner and disable ""On-Screen Display support""");
            }
        }

        public override void OnShutdown()
        {
            Task.Run(OnDisableAsync);
        }

        public override void OnEnabled()
        {
            Task.Factory.StartNew(RunRenderForm, TaskCreationOptions.LongRunning);
        }

        private void RunRenderForm()
        {
            OverlayManager.Drawing += Drawing;

            IntPtr targetWindow = Core.Memory.Process.MainWindowHandle;

            _renderForm = new RenderForm(targetWindow);

            Application.Run(_renderForm);
        }

        public override void OnDisabled()
        {
            Task.Run(OnDisableAsync);
        }

       

        private async Task OnDisableAsync()
        {
            OverlayManager.Drawing -= Drawing;

            if (_renderForm == null)
                return;

            await _renderForm.ShutdownAsync();
        }

        public override string Name => "3D Overlay";
        public override string Description => "Reborn Buddy 3d Overlay";

        public override string Author => "The Buddy Team & ZZI";

        public override Version Version => new Version(1, 0, 0);

        public override string ButtonText => "Settings";

        public override bool WantButton => true;

        private SettingsForm _settingsForm;
        public override void OnButtonPress()
        {
            if (_settingsForm == null)
                _settingsForm = new SettingsForm();

            _settingsForm.ShowDialog();
        }


        /// <summary>
        /// Converts a screen space point into a corresponding point in world space.
        /// </summary>
        /// <param name="source">The vector to project.</param>
        /// <param name="matrix">An inverted combined WorldViewProjection matrix.</param>
        /// <param name="vector">The unprojected vector.</param>
        public Vector3 Unproject(Vector3 source, Viewport vp, SlimDX.Matrix matrix)
        {
            return Vector3.Unproject(source, vp.X, vp.Y, vp.Width, vp.Height, vp.MinZ, vp.MaxZ, matrix);
        }

        Color myaqua = Color.FromArgb(75, Color.Aqua);
        private void Drawing(DrawingContext ctx)
        {
            OverlaySettings settings = OverlaySettings.Instance;

            if (settings.OnlyDrawInForeground &&
                Imports.GetForegroundWindow() != Core.Memory.Process.MainWindowHandle)
                return;

            if (QuestLogManager.InCutscene)
                return;

            //Gameobject list is threadstatic, always need to pulse otherwise new objects wont show up
            GameObjectManager.Update();

            NVector3 mypos = Core.Me.Location;
            Vector3 vecStart = new Vector3(mypos.X, mypos.Y, mypos.Z);
            int myLevel = Core.Me.ClassLevel;


            if (settings.DrawGameStats)
            {
                StringBuilder sb = new StringBuilder();

                GameObject currentTarget = Core.Me.CurrentTarget;


                sb.AppendLine("My Position: " + Core.Me.Location);
                if (currentTarget != null)
                {
                    sb.AppendLine("Current Target: " + currentTarget.Name + ", Distance: " +
                                  Math.Round(currentTarget.Distance(), 3));

                    NVector3 end = currentTarget.Location;
                    Vector3 vecEnd = new Vector3(end.X, end.Y, end.Z);

                    ctx.DrawLine(vecStart, vecEnd, Color.DeepSkyBlue);
                }
                else
                {
                    sb.AppendLine("");
                }
                sb.AppendLine("");
                sb.AppendLine(string.Format("XP Per Hour: {0:F0}", GameStatsManager.XPPerHour));
                sb.AppendLine(string.Format("Deaths Per Hour: {0:F0}", GameStatsManager.DeathsPerHour));

                if (myLevel < 90)
                    sb.AppendLine(string.Format("Time to Level: {0}", GameStatsManager.TimeToLevel));

                sb.AppendLine(string.Format("TPS: {0:F2}", GameStatsManager.TicksPerSecond));

                sb.AppendLine();

                if (settings.UseShadowedText)
                {
                    ctx.DrawOutlinedText(sb.ToString(),
                                         settings.GameStatsPositionX,
                                         settings.GameStatsPositionY,
                                         settings.GameStatsForegroundColor,
                                         settings.GameStatsShadowColor,
                                         settings.GameStatsFontSize
                        );
                }
                else
                {
                    ctx.DrawText(sb.ToString(),
                                 settings.GameStatsPositionX,
                                 settings.GameStatsPositionY,
                                 settings.GameStatsForegroundColor,
                                 settings.GameStatsFontSize
                        );
                }

                ctx.DrawOutlinedBox(Core.Me.Location.Convert() + new Vector3(0, 1, 0), new Vector3(1),
                    Color.FromArgb(255, Color.Blue));
            }

            if (settings.HideBehindGameWindows)
            {
                ctx.Device.SetTransform(TransformState.View, Matrix.Identity);
                ctx.Device.SetRenderState(RenderState.ColorWriteEnable, (ColorWriteEnable)0);

                var matrix = FFXIVCameraEx.Projection * Matrix.Identity * Matrix.Identity;

                foreach (var window in RaptureAtkUnitManager.Controls)
                {
                    if (!window.IsVisible)
                        continue;

                    if (!settings.WindowsToIgnore.Any(r=> window.Name.Contains(r)))
                    {

                        var _bounds = window.Bounds;

                        if (!_bounds.HasValue)
                            continue;

                        var bounds = _bounds.Value;

                        if (bounds.Width >= ctx.Device.Viewport.Width || bounds.Height >= ctx.Device.Viewport.Height)
                            continue;

                        var x = bounds.X;
                        var z = bounds.Y;
                        var width = bounds.Width;
                        var height = bounds.Height;

                        var a = Unproject(new Vector3(x, z, 0),ctx.Device.Viewport,matrix);
                        var b = Unproject(new Vector3(x + width, z, 0), ctx.Device.Viewport, matrix);
                        var c = Unproject(new Vector3(x + width, z + height, 0), ctx.Device.Viewport, matrix);
                        var d = Unproject(new Vector3(x, z + height, 0), ctx.Device.Viewport, matrix);
                        
                        ctx.DrawRectangle(new List<Vector3>() { a, b, c, d }, myaqua);

                        //ctx.DrawText(window.Name, x, z, Color.Black);
                    }
                }

                ctx.Device.SetRenderState(RenderState.ColorWriteEnable, ColorWriteEnable.All);
                ctx.Device.SetTransform(TransformState.View, FFXIVCameraEx.View);
            }


            if (settings.DrawHostilityBoxes || settings.DrawUnitLines || settings.DrawGameObjectBoxes || settings.DrawGameObjectLines)
            {

                foreach (GameObject obj in GameObjectManager.GameObjects)
                {



                    if (!obj.IsVisible)
                        continue;

                    if (settings.OnlyRenderTargetable)
                    {
                        if (obj.Type != GameObjectType.EventObject)
                        {
                            if (!obj.IsTargetable)
                                continue;
                        }
                    }



                    if (obj.Type == GameObjectType.Mount)
                        continue;

                    var name = obj.Name;
                    var vecCenter = obj.Location.Convert() + new Vector3(0,1,0);


                    //.Where(i => i.Type == GameObjectType.GatheringPoint || i.Type == GameObjectType.BattleNpc || i.Type == GameObjectType.EventObject || i.Type == GameObjectType.Treasure || i.Type == GameObjectType.Pc)



                    var color = Color.FromArgb(150, Color.Blue);

                    //some generic objects. If you want to add a specific object it should probably go here or in it's own block below this.
                    if (obj.Type == GameObjectType.GatheringPoint || obj.Type == GameObjectType.EventObject || obj.Type == GameObjectType.Treasure)
                    {
                        if (obj.Type == GameObjectType.GatheringPoint)
                            color = Color.FromArgb(150, Color.BlueViolet);
                        if (obj.Type == GameObjectType.EventObject)
                            color = Color.FromArgb(150, Color.Fuchsia);
                        if (obj.Type == GameObjectType.Treasure)
                            color = Color.SandyBrown;

                        if (settings.DrawGameObjectNames && !string.IsNullOrEmpty(name))
                            ctx.Draw3DText(name, vecCenter);

                        if (settings.DrawGameObjectBoxes)
                        {
                            ctx.DrawOutlinedBox(vecCenter, new Vector3(1),Color.FromArgb(150, color));
                        }




                        //if (settings.DrawGameObjectLines)
                        //{
                        //    if (!settings.DrawGameObjectLinesLos || obj.InLineOfSight())
                        //        ctx.DrawLine(vecStart, vecCenter, Color.FromArgb(150, color));
                        //}





                    }

                    var u = obj as Character;
                    if (u != null)
                    {


                        var playerOrPlayerOwned = (!u.IsNpc || u.SummonerObjectId != GameObjectManager.EmptyGameObject);
                        if (!settings.DrawPlayers && playerOrPlayerOwned)
                        {
                            continue;
                        }

                        var hostilityColor = Color.FromArgb(150, Color.Green);

                        var uStatusFlags = u.StatusFlags;
                        if (uStatusFlags.HasFlag(StatusFlags.Hostile))
                        {
                            hostilityColor = Color.FromArgb(150, Color.Red);

                            //if (settings.DrawAggroRangeCircles)
                            //    ctx.DrawCircle(vecCenter, u.MyAggroRange, 64,
                            //                   Color.FromArgb(75, Color.DeepSkyBlue));
                        }

                        if (uStatusFlags == StatusFlags.None)
                            hostilityColor = Color.FromArgb(150, Color.Yellow);

                        if (uStatusFlags.HasFlag(StatusFlags.Friend) || uStatusFlags.HasFlag(StatusFlags.PartyMember) || uStatusFlags.HasFlag(StatusFlags.AllianceMember))
                            hostilityColor = Color.FromArgb(150, Color.Green);




                        if (playerOrPlayerOwned)
                        {
                            if (settings.DrawPlayerNames)
                            {
                                ctx.Draw3DText(name, vecCenter);
                            }
                        }
                        else
                        {
                            if (settings.DrawUnitNames)
                            {
                                if (!string.IsNullOrEmpty(name) && obj.IsTargetable)
                                    ctx.Draw3DText(name, vecCenter);
                            }
                        }



                        ctx.DrawOutlinedBox(vecCenter, new Vector3(1), Color.FromArgb(255, hostilityColor));
                    }
                }
            }

           
            //Profile curProfile = ProfileManager.CurrentProfile;
            //if (curProfile != null)
            //{
            //    if (settings.DrawHotspots)
            //    {
            //        GrindArea ga = QuestState.Instance.CurrentGrindArea;
            //        if (ga != null)
            //        {
            //            if (ga.Hotspots != null)
            //            {
            //                foreach (Hotspot hs in ga.Hotspots)
            //                {
            //                    var p = hs.Position;
            //                    Vector3 vec = new Vector3(p.X, p.Y, p.Z);
            //                    ctx.DrawCircle(vec, 10.0f, 32, Color.FromArgb(200, Color.Red));

            //                    if (!string.IsNullOrWhiteSpace(hs.Name))
            //                        ctx.Draw3DText("Hotspot: " + hs.Name, vec);
            //                    else
            //                    {
            //                        ctx.Draw3DText("Hotspot", vec);
            //                    }
            //                }
            //            }
            //        }

            //        // This is only used by grind profiles.
            //        if (curProfile.HotspotManager != null)
            //        {
            //            foreach (NVector3 p in curProfile.HotspotManager.Hotspots)
            //            {
            //                Vector3 vec = new Vector3(p.X, p.Y, p.Z);
            //                ctx.DrawCircle(vec, 10.0f, 32, Color.FromArgb(200, Color.Red));
            //                ctx.Draw3DText("Hotspot", vec);
            //            }
            //        }
            //    }

            //    if (settings.DrawBlackspots)
            //    {
            //        foreach (Blackspot bs in BlackspotManager.GetAllCurrentBlackspots())
            //        {
            //            var p = bs.Location;
            //            Vector3 vec = new Vector3(p.X, p.Y, p.Z);
            //            ctx.DrawCircle(vec, bs.Radius, 32, Color.FromArgb(200, Color.Black));

            //            if (!string.IsNullOrWhiteSpace(bs.Name))
            //                ctx.Draw3DText("Blackspot: " + bs.Name, vec);
            //            else
            //            {
            //                ctx.Draw3DText("Blackspot: " + vec, vec);
            //            }
            //        }
            //    }
            //}
        }
    }
}
